
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "complex.h"

COMPLEX cx_series( COMPLEX a, COMPLEX b)
{
   return cx_add( a, b);
}

COMPLEX cx_parallel( COMPLEX a, COMPLEX b)
{
   return cx_div( cx_mul( a, b), cx_add( a, b));
}

COMPLEX cx_cap( double w, double c)
{
    return (COMPLEX){ 0.0, -1.0/( w * c)};
}

COMPLEX cx_ind( double w, double l)
{
   return (COMPLEX){ 0.0, w * l};
}

COMPLEX cx_exp( COMPLEX a)
{
   COMPLEX s, r;

   s.re = cos( a.im);
   s.im = sin( a.im);

   r.re = exp( a.re);
   r.im = 0.0;

   return cx_mul( s, r);
}

COMPLEX cx_cosh( COMPLEX a)
{
   COMPLEX s, b;

   b.re = -a.re;
   b.im = -a.im;

   s = cx_add( cx_exp( a), cx_exp( b));
   s.re /= 2.0;
   s.im /= 2.0;

   return s;
}

COMPLEX cx_sinh( COMPLEX a)
{
   COMPLEX s, b;

   b.re = -a.re;
   b.im = -a.im;

   s = cx_sub( cx_exp( a), cx_exp( b));
   s.re /= 2.0;
   s.im /= 2.0;

   return s;
}

COMPLEX cx_tanh( COMPLEX a)
{
   return cx_div( cx_sinh( a), cx_cosh( a));
}

COMPLEX cx_sqrt( COMPLEX a)
{
   COMPLEX s;
   double arg = cx_arg( a);
   double abs = cx_mag( a);

   s.re = sqrt( abs) * cos( arg/2.0);   
   s.im = sqrt( abs) * sin( arg/2.0);

   return s;
}

