struct S_COMPLEX 
{
   double re;
   double im;
};

typedef struct S_COMPLEX COMPLEX;

static inline COMPLEX cx_add( COMPLEX a, COMPLEX b)
{
   COMPLEX t;

   t.re = a.re + b.re;
   t.im = a.im + b.im;

   return t;
}

static inline COMPLEX cx_sub( COMPLEX a, COMPLEX b)
{
   COMPLEX t;

   t.re = a.re - b.re;
   t.im = a.im - b.im;

   return t;
}

static inline COMPLEX cx_scale( COMPLEX a, double r)
{
   return (COMPLEX){a.re * r, a.im * r};
}

static inline COMPLEX cx_neg( COMPLEX a)
{
   return (COMPLEX){-a.re, -a.im};
}

static inline COMPLEX cx_mul( COMPLEX a, COMPLEX b)
{
   COMPLEX t;

   t.re = a.re * b.re - a.im * b.im;
   t.im = a.im * b.re + a.re * b.im;

   return t;
}

static inline double cx_mag( COMPLEX a)
{
   return sqrt( a.re * a.re + a.im * a.im);
}

static inline COMPLEX cx_div( COMPLEX a, COMPLEX b)
{
   COMPLEX t;
   double r;
   
   t = cx_mul( a, (COMPLEX){ b.re, -b.im});
   r = b.re * b.re + b.im * b.im;

   t.re /= r;
   t.im /= r;

   return t;
}

static inline double cx_arg( COMPLEX a)
{
   return atan2( a.im, a.re); 
}

COMPLEX cx_sub( COMPLEX a, COMPLEX b);
void cx_fprint( FILE *f, COMPLEX a);
COMPLEX cx_mul( COMPLEX a, COMPLEX b);
COMPLEX cx_div( COMPLEX a, COMPLEX b);
double cx_mag( COMPLEX a);

COMPLEX cx_series( COMPLEX a, COMPLEX b);
COMPLEX cx_parallel( COMPLEX a, COMPLEX b);

COMPLEX cx_cap( double w, double c);
COMPLEX cx_ind( double w, double l);

COMPLEX cx_sqrt( COMPLEX a);
COMPLEX cx_tanh( COMPLEX a);
COMPLEX cx_exp( COMPLEX a);
COMPLEX cx_sin( COMPLEX a);
COMPLEX cx_cos( COMPLEX a);

COMPLEX cx_neg( COMPLEX a);

