/*
 *  tsim-util.c 
 *
 *  author: paul@abelian.demon.co.uk
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <string.h>
#include <math.h>
#include <unistd.h>

#include "complex.h"

extern int use_lundin, use_palermo, Nt, Ns, Ncp;
extern double Lcoil, Dp, Wrad, Wrad, Ctop, RcoilDC, Crad, Hcoil,
              spacing_ratio, Cext[], Ctor[], Cint[ 100][ 100];

extern char sysname[];

int NCP;

FILE *fdi;

void logmsg( char *format, ...)
{
   FILE *fl;
   char temp[ 500];

   sprintf( temp, "%s.log", sysname);
   if( (fl = fopen( temp, "a+")) == NULL) return;

   vsprintf( temp, format, &format + 1);

   fprintf( fl, "%s\n", temp);
   fclose( fl);

   printf( "%s\n", temp);
}

void *safe_malloc( int size)
{
   void *p = malloc( size);
 
   if( !p)
   {
      logmsg( "not enough memory [%d]", size);
      perror( "not enough memory");
      exit( 1);
   }

   return p;
}

/*****************************************************************************/
/* Input File Routines                                                       */
/*****************************************************************************/

char pbuff[ 500], *pb;
int lino;
int eof = 0;

int parse_reload( void)
{
   *(pb = pbuff) = 0;

   if( fgets( pbuff, 499, fdi)) { lino++; return 1; }

   eof = 1;
   return 0;
}

void parse_error( char *msg)
{
   fprintf( stderr, "error in input file: line %d: %s\n", lino, msg);
   exit( 1);
}

void skip_whitespace( void)
{
   while( 1)
   {
      if( !*pb)
         if( !parse_reload()) return;
         else continue;
     
      switch( *pb)
      {
         case 32:      
         case '\t':
         case '\n':   
         case '\r': pb++; continue;

         case ';': *pb = 0; continue;

         default: return;
      }
   }
}

int read_word( char *s)
{
   int qflag = 0;
   int n = 0;

   skip_whitespace();

   while( *pb || qflag)
   {
      if( qflag)
      {
         if( !*pb || *pb == '\r' || *pb == '\n')
            parse_error( "unterminated string");
         if( *pb == '"')
         {
            pb++; 
            break;
         }
      }
      else
      {
         if( *pb == 32 || *pb == '\t' || *pb == '\n' || *pb == '\r') break;
         if( !n && *pb == '"') { pb++; qflag = 1; continue; }
      }

      *s++ = *pb++; n++;
   }

   *s = 0;
   return n;
}

int parse_word( char *target)
{
   int n = strlen( target);

   skip_whitespace();
   if( strncasecmp( pb, target, n)) return 0;

   pb += n;
   return 1;
}

int parse_toroid( void)
{
   char temp[ 100];

   if( !parse_word( "{")) parse_error( "missing { after TOROID");

   while( !parse_word( "}")) 
   {
      if( eof) return 0;
      read_word( temp);
   }

   return 1;
}

int parse_disc( void)
{
   char temp[ 100];

   if( !parse_word( "{")) parse_error( "missing { after DISC");

   while( !parse_word( "}")) 
   {
      if( eof) return 0;
      read_word( temp);
   }

   return 1;
}

int parse_coil( void)
{
   char temp[ 100];

   if( !parse_word( "{")) parse_error( "missing { after COIL");

   while( !parse_word( "}")) 
   {
      if( eof) return 0;

      if( parse_word( "name"))
      {
         read_word( temp);
      }
      else
      if( parse_word( "radius"))
      {
         read_word( temp); Crad = atof( temp);
      }
      else
      if( parse_word( "length"))
      {
         read_word( temp); Hcoil += atof( temp);
      }
      else
      if( parse_word( "inductance"))
      {
         read_word( temp);  Lcoil = atof( temp);
      }
      else
      if( parse_word( "conductor_radius"))
      {
         read_word( temp);  Wrad = atof( temp);
      }
      else
      if( parse_word( "resistance"))
      {
         read_word( temp);  RcoilDC += atof( temp);
      }
      else
      if( parse_word( "altitude"))
      {
         read_word( temp);
      }
      else
      if( parse_word( "turns"))
      {
         read_word( temp); Nt += atoi( temp);
      }
      else
      if( parse_word( "use_lundin")) use_lundin = 1;
      else parse_error( "unrecognised COIL parameter");
   }

   return 1;
}

int parse_ground_plane( void)
{
   if( !parse_word( "{") || !parse_word( "}"))
      parse_error( "missing {} after GROUND_PLANE");
   return 1;
}

int parse_begin( void)
{
   if( !parse_word( "BEGIN"))
   {
      fprintf( stderr, "missing BEGIN\n");
      return 0;
   }

   while( 1)
   {
      char temp[ 50];

      if( eof) { parse_error( "unexpected end of file"); return 0; }
      if( parse_word( "END")) return 1;
      else
      if( parse_word( "NCP")){ read_word( temp); NCP = atoi( temp); }
      else
      if( parse_word( "TOROID") && !parse_toroid()) return 0;
      else
      if( parse_word( "DISC") && !parse_disc()) return 0;
      else
      if( parse_word( "COIL") && !parse_coil()) return 0;
      else
      if( parse_word( "GROUND_PLANE") && !parse_ground_plane()) return 0;
   }

   return parse_word( "END");
}

void parse_input_file( void)
{
   char temp[ 100];
   pbuff[ 0] = 0; pb = pbuff;
   eof = lino = 0;

   NCP = 0;

   sprintf( temp, "%s.in", sysname);
   if( (fdi = fopen( temp, "r")) == NULL)
   {
      fprintf( stderr, "cannot open input file %s\n", temp);
      perror( "");
      exit( 1);
   }
  
   if( !parse_begin()) exit( 1);

   fclose( fdi);
}

int parse_caps_begin( void)
{
   int n1, n2;
   char temp[ 50];

   while( !eof)
   {
      if( parse_word( "external"))
      {
         if( parse_word( "toroid"))
         {
            read_word( temp);
            read_word( temp); Ctop += 2 * atof( temp) * 1e-12;
         }
         else
         if( parse_word( "coil"))
         {
            read_word( temp);  n1 = atoi( temp); 
            read_word( temp); Cext[ n1] += 2 * atof( temp) * 1e-12;
            if( n1 > Ncp) Ncp = n1;
         }
         else
         if( parse_word( "disc"))
         {
            read_word( temp);  
            read_word( temp);  
         }
      }
      else
      if( parse_word( "internal"))
      {
         if( parse_word( "disc"))
         {
            read_word( temp);  
            read_word( temp);   /* to */
            if( parse_word( "coil"))
            {
               read_word( temp);   n1 = atoi( temp);
               read_word( temp); Cext[ n1] += atof( temp) * 1e-12;
            }
            else
            if( parse_word( "toroid"))
            {
               read_word( temp);   
               read_word( temp); Ctop += atof( temp) * 1e-12;
            }
            else
            if( parse_word( "disc"))
            {
               read_word( temp);   
               read_word( temp);   
            }
         }
         else
         if( parse_word( "toroid"))
         {
            read_word( temp);  
            read_word( temp);   /* to */
            if( parse_word( "disc"))
            {
               read_word( temp);   
               read_word( temp); Ctop += atof( temp) * 1e-12;
            }
            else
            if( parse_word( "coil"))
            {
               read_word( temp);   n1 = atoi( temp);
               read_word( temp); Ctor[ n1] += atof( temp) * 1e-12;
            }
            else
            if( parse_word( "toroid"))
            {
               read_word( temp);   
               read_word( temp);   
            }
         }
         else
         if( parse_word( "coil"))
         {
            read_word( temp);  n1 = atoi( temp);
            read_word( temp);   /* to */
            if( parse_word( "coil"))
            {
               read_word( temp);   n2 = atoi( temp);
               read_word( temp); Cint[ n1][ n2] += atof( temp) * 1e-12;
            }
            else
            if( parse_word( "toroid"))
            {
               read_word( temp);
	       read_word( temp); Ctor[ n1] += atof( temp) * 1e-12;
            }
            else
            if( parse_word( "disc"))
            {
               read_word( temp);
	       read_word( temp); Cext[ n1] += atof( temp) * 1e-12;
            }
         }
      }
      else if( !eof) { parse_error( "error in caps file"); return 0; }
   }

   return 1;
}

void parse_caps_file( void)
{
   int i, j;
   char temp[ 100];
   pbuff[ 0] = 0; pb = pbuff;
   eof = lino = 0;

   sprintf( temp, "%s.caps", sysname);
   if( (fdi = fopen( temp, "r")) == NULL)
   {
      fprintf( stderr, "cannot open caps file %s\n", temp);
      perror( "");
      exit( 1);
   }
  
   if( !parse_caps_begin()) exit( 1);

   fclose( fdi);

   Ctop /= 2; if( Ctop == 0.0) Ctop = 0.1e-12;
   for( i=1; i<=Ncp; i++) Cext[ i] /= 2;
   for( i=1; i<=Ncp; i++) Ctor[ i] /= 2;
   for( i=1; i<Ncp; i++) 
      for( j=i+1; j<=Ncp; j++)
      {
         double a = (Cint[ i][ j] + Cint[ j][ i]);
         Cint[ i][ j] = Cint[ j][ i] = a/2;
      }
}

void load_system_description( void)
{
   int i, j;

   for( i=0; i<100; i++) 
   {
      Cext[ i] = Ctor[ i] = 0.0;
      for( j=0; j<100; j++) Cint[ i][ j] = 0.0;
   }
   parse_input_file();
   parse_caps_file();
}

